<?php
require_once("config.php");
$table = "events";
header("Access-Control-Allow-Orgin: *");
header("Access-Control-Allow-Methods: *");
header("Content-Type: application/json");
$requestUri = explode("/", trim($_SERVER["REQUEST_URI"],"/"));
$requestParams = $_REQUEST;

$method = $_SERVER["REQUEST_METHOD"];

try {
	$db = new PDO($dsn, $username, $password);
	$status = "200";
	$statusText = "OK";

	switch ($method) {
		case "GET":
			$queryParams = [];
			$queryText = "SELECT * FROM `".$table."`";
			if (isset($requestParams["from"]) && isset($requestParams["to"])) {
				$queryText .= " WHERE `end_date`>=? AND `start_date` < ?;";
				$queryParams = [$params["from"], $params["to"]];
			}
			$query = $db->prepare($queryText);
			$query->execute($queryParams);
			$result = $query->fetchAll();
		break;
		case "POST":
			$requestPayload = json_decode(file_get_contents("php://input"));
			$id = $requestPayload->id;
			$action = $requestPayload->action;
			$body = (array) $requestPayload->data;
			$statusText = $action;
			if ($action == "inserted") {
				$queryText = "INSERT INTO `".$table."` SET
				`start_date`=?,
				`end_date`=?,
				`text`=?,
				 `event_pid`=?,
				`event_length`=?,
				`rec_type`=?";
				$queryParams = [
					$body["start_date"],
					$body["end_date"],
					$body["text"],
					// recurring events columns
					$body["event_pid"] ? $body["event_pid"] : 0,
					$body["event_length"] ? $body["event_length"] : 0,
					$body["rec_type"]
				];

				// delete a single occurrence from  recurring series
				$resultAction = "inserted";
				if ($body["rec_type"] === "none") {
					$resultAction = "deleted";//!
				}
				/*
				end of recurring events data processing
				*/

				$query = $db->prepare($queryText);
				$query->execute($queryParams);
				$result = [
					"tid" => $db->lastInsertId(),
					"action" => $resultAction
				];
			} elseif($action == "updated") {
				$queryText = "UPDATE `".$table."` SET
				`start_date`=?,
				`end_date`=?,
				`text`=?,
				`event_pid`=?,
				`event_length`=?,
				`rec_type`=?
				WHERE `id`=?";

				$queryParams = [
					$body["start_date"],
					$body["end_date"],
					$body["text"],

					$body["event_pid"] ? $body["event_pid"] : 0,
					$body["event_length"] ? $body["event_length"] : 0,
					$body["rec_type"],//!
					$id
				];
				if ($body["rec_type"] && $body["rec_type"] != "none") {
					//all modified occurrences must be deleted when you update recurring series
					//https://docs.dhtmlx.com/scheduler/server_integration.html#savingrecurringevents
					$subQueryText = "DELETE FROM `".$table."` WHERE `event_pid`=? ;";
					$subQuery = $db->prepare($subQueryText);
					$subQuery->execute([$id]);
				}

				$query = $db->prepare($queryText);
				$query->execute($queryParams);

				$result = [
					"action" => "updated"
				];
			} elseif($action == "deleted") {
				// some logic specific to recurring events support
				// https://docs.dhtmlx.com/scheduler/server_integration.html#savingrecurringevents
				$subQueryText = "SELECT * FROM `events` WHERE id=? LIMIT 1;";
				$subQuery = $db->prepare($subQueryText);
				$subQuery->execute([$id]);
				$event = $subQuery->fetch(PDO::FETCH_ASSOC);

				if ($event["event_pid"]) {
					// deleting a modified occurrence from a recurring series
					// If an event with the event_pid value was deleted - it needs updating
					// with rec_type==none instead of deleting.
					$subQueryText="UPDATE `".$table."` SET `rec_type`='none' WHERE `id`=?;";
					$subQuery = $db->prepare($subQueryText);
					$query->execute($queryParams);
					$result = [
						"action" => "deleted"
					];
				}

				if ($event["rec_type"] && $event["rec_type"] != "none") {//!
					// if a recurring series deleted, delete all modified occurrences of the series
					$subQueryText = "DELETE FROM `".$table."` WHERE `event_pid`=? ;";
					$subQuery = $db->prepare($subQueryText);
					$subQuery->execute([$id]);
				}

				/*
				end of recurring events data processing
				*/

				$queryText = "DELETE FROM `".$table."` WHERE `id`=? ;";

				$query = $db->prepare($queryText);
				$query->execute([$id]);

				$result = [
					"action" => "deleted"
				];
			}
		break;
		default: throw new Exception("Unexpected Method"); break;
	}
} catch (Exception $e) {
	$status = "500";
	$statusText = "Server error";
	$result = [
		"action" => "error",
		"message" => $e->getMessage()
	];
}
header("HTTP/1.1 " . $status . " " . $statusText);
$payload = json_encode($result);
echo $payload;